# © 2022 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

import base64

from odoo import SUPERUSER_ID, _, fields, models
from odoo.exceptions import UserError


class ScopContributionReport(models.Model):
    _inherit = "scop.contribution.report"

    has_bordereau = fields.Boolean("Bordereau lié", compute="_compute_has_bordereau")

    # ------------------------------------------------------
    # Compute
    # ------------------------------------------------------
    def _compute_has_bordereau(self):
        contrib_cg = self.env.ref("cgscop_partner.riga_14397")
        for contribution in self:
            # exclude contribution before 2021 from RIGA
            if contribution.year > "2020":
                if contribution.type_contribution_id == contrib_cg:
                    bordereau = (
                        self.env["scop.bordereau"]
                        .sudo()
                        .search(
                            [
                                ("partner_id", "=", contribution.partner_id.id),
                                ("year", "=", contribution.year),
                            ]
                        )
                    )
                    if len(bordereau) == 1:
                        contribution.has_bordereau = True
                    else:
                        contribution.has_bordereau = False
                else:
                    contribution.has_bordereau = False
            else:
                contribution.has_bordereau = False

    # ------------------------------------------------------
    # Button / Action
    # ------------------------------------------------------
    def print_report_with_payment(self):
        bordereau = (
            self.env["scop.bordereau"]
            .sudo()
            .search(
                [
                    ("partner_id", "=", self.partner_id.id),
                    ("year", "=", self.year),
                ]
            )
        )
        if len(bordereau) == 1:
            return (
                self.env.ref(
                    "cgscop_cotisation_cg.cgscop_bordereau_report_with_payments"
                )
                .sudo()
                .report_action(bordereau.sudo())
            )
        else:
            raise UserError(_("Pas de bordereau rattaché à cette cotisation."))

    def print_contribution(self):
        self.ensure_one()
        if self.has_bordereau:
            bordereau = (
                self.env["scop.bordereau"]
                .sudo()
                .search(
                    [
                        ("partner_id", "=", self.partner_id.id),
                        ("year", "=", self.year),
                        ("state", "not in", ["draft", "cancel"]),
                    ]
                )
            )
            if len(bordereau) == 1:
                report_sudo = self.env.ref(
                    "cgscop_cotisation_cg.cgscop_bordereau_report"
                ).sudo()
                bordereau_pdf = report_sudo.with_user(SUPERUSER_ID)._render_qweb_pdf(
                    bordereau.id
                )[0]
                return base64.b64encode(bordereau_pdf)
            else:
                raise UserError(
                    _("Impossible de télécharger le bordereau associé à cette cotisation.")
                )
        else:
            raise UserError(_("Pas de bordereau rattaché à cette cotisation."))

    # ------------------------------------------------------
    # Business method
    # ------------------------------------------------------
    def get_invoice_contribution(self):
        """
        If cotiz is type CG, then we filtered on not canceled bordereaux
        """
        self.ensure_one()
        invoice_ids = super(ScopContributionReport, self).get_invoice_contribution()
        contribution_cg = self.env.ref("cgscop_partner.riga_14397")
        if self.type_contribution_id == contribution_cg:
            invoice_ids = invoice_ids.filtered(
                lambda i: i.bordereau_id.state not in ("cancel",)
            )
        return invoice_ids

    # ------------------------------------------------------
    # Inherit parent
    # ------------------------------------------------------
    def _get_contribution_domain(self):
        """
        Hérite la fonction parente pour ajouter la notion de bordereau
        dans la vue pour les cotisations CG
        """ ""
        domain = super()._get_contribution_domain()
        add_domain = [
            "|",
            ("bordereau_id.state", "not in", ("cancel", "new")),
            ("bordereau_id", "=", False),
        ]
        domain += add_domain
        return domain

    def _where_invoice(self):
        """
        Hérite la fonction parente pour ne prendre que les factures/avoirs rattachés à
        un bordereau pour la CG Scop et toutes pour les autres sociétés
        """
        where_str = super()._where_invoice()
        where_str += """
            AND (
                (i.company_id = 1 AND i.bordereau_id is not null) OR (i.company_id != 1)
            )
        """
        return where_str
