# Copyright 2020 Le Filament
# License AGPL-3.0 or later (https://www.gnu.org/licenses/agpl).

from odoo import _, api, fields, models
from odoo.exceptions import UserError, ValidationError


class ScopCotisationRegul(models.TransientModel):
    _name = "scop.cotisation.regul.wizard"
    _description = "Regularisation cotisation CG"

    name = fields.Char()
    date_regul = fields.Date(
        string="Date de régularisation", default=fields.Date.today()
    )
    bordereau_id = fields.Many2one(
        comodel_name="scop.bordereau",
        string="Bordereau",
        readonly=True,
    )
    partner_id = fields.Many2one(
        comodel_name="res.partner",
        string="Adhérent",
        readonly=True,
    )

    # Old liasse
    liasse_fiscale_id = fields.Many2one(
        comodel_name="scop.liasse.fiscale",
        string="Liasse Fiscale de référence",
    )
    year = fields.Integer("Année de la liasse")
    type_assiette_retenu = fields.Selection(
        string="Type assiette",
        selection=[
            ("ca", "CA"),
            ("va", "VA"),
        ],
    )
    montant_assiette = fields.Float(
        string="Montant assiette de cotisation",
    )
    amount_total_cotiz = fields.Float(string="Montant total de(s) cotisation(s)")
    detail = fields.Text("Détail cotisation")
    comment = fields.Char("Motif de régularisation")

    # New liasse
    liasse_fiscale_new_id = fields.Many2one(
        comodel_name="scop.liasse.fiscale",
        string="Liasse Fiscale",
    )
    type_id_new = fields.Selection(
        string="Type de liasse", related="liasse_fiscale_new_id.type_id"
    )
    source_new = fields.Selection(
        string="Source de la liasse", related="liasse_fiscale_new_id.source"
    )
    type_assiette_new = fields.Selection(
        related="liasse_fiscale_new_id.contribution_base_type"
    )
    montant_assiette_new = fields.Integer(
        related="liasse_fiscale_new_id.contribution_base_amount"
    )
    amount_cg = fields.Float(related="liasse_fiscale_new_id.contribution_cg")
    amount_ur_med = fields.Float(related="liasse_fiscale_new_id.contribution_med")
    amount_ur_hdf = fields.Float(related="liasse_fiscale_new_id.contribution_hdf")
    amount_fede_com = fields.Float(related="liasse_fiscale_new_id.contribution_com")
    amount_fede_cae = fields.Float(related="liasse_fiscale_new_id.contribution_cae")

    is_payment = fields.Boolean("Paiements liés")

    # ------------------------------------------------------
    # Constrains
    # ------------------------------------------------------
    @api.constrains("date_regul")
    def _check_date_regul(self):
        invoice_date = self.bordereau_id.invoice_ids.mapped("invoice_date")
        if invoice_date:
            last_date = max(invoice_date)
        else:
            last_date = self.bordereau_id.base_cotisation_cg.date_cotisation
        if self.date_regul > fields.Date.today() or self.date_regul < last_date:
            raise ValidationError(
                _(
                    "La date de régularisation doit être "
                    "inférieure ou égale à la date du jour et "
                    "supérieure à la dernière date de "
                    "facturation liée au bordereau."
                )
            )

    # ------------------------------------------------------
    # Override ORM
    # ------------------------------------------------------
    @api.model
    def default_get(self, fields):
        res = super(ScopCotisationRegul, self).default_get(fields)
        bordereau_id = self.env["scop.bordereau"].browse(
            self.env.context.get("active_id")
        )
        contribs = bordereau_id.invoice_ids.read_group(
            [("id", "in", bordereau_id.invoice_ids.ids)],
            ["type_contribution_id", "amount_total_signed"],
            ["type_contribution_id"],
        )
        detail = "<table class='o_group o_inner_group'>"
        for contrib in contribs:
            detail += (
                '<tr><td class="o_td_label font-weight-bold">'
                + str(contrib.get("type_contribution_id")[1])
                + '</td><td style="width: 100%;">'
                + str(contrib.get("amount_total_signed"))
                + "€</td>"
            )
        detail += "</table>"
        payments = list(
            map(
                lambda i: True if i.move_line_payment_ids else False,
                bordereau_id.invoice_ids,
            )
        )
        if True in payments:
            is_payment = True
        else:
            is_payment = False
        res.update(
            {
                "bordereau_id": bordereau_id.id,
                "partner_id": bordereau_id.partner_id.id,
                "liasse_fiscale_id": bordereau_id.liasse_fiscale_id.id,
                "year": bordereau_id.year_liasse,
                "type_assiette_retenu": bordereau_id.type_assiette,
                "montant_assiette": bordereau_id.montant_assiette,
                "amount_total_cotiz": bordereau_id.amount_total_cotiz,
                "detail": detail,
                "is_payment": is_payment,
            }
        )
        return res

    # ------------------------------------------------------
    # Action
    # ------------------------------------------------------
    def update_contribution(self):
        if not self.comment:
            raise UserError(
                _("Vous devez renseigner un motif de " "régularisation pour valider.")
            )

        self.bordereau_id.update_bordereau_version(
            date_regul=self.date_regul,
            comment=self.comment,
            liasse_fiscale_new_id=self.liasse_fiscale_new_id,
        )
